@@ -14,8 +14,9 @@ from typing import Dict, List, Tuple
 
 import codecs
 import csv
+import json
 
-from flowserv.model.files.base import FileHandle
+from flowserv.model.files import FileHandle
 from flowserv.service.api import APIFactory
 
 
@@ -70,12 +71,22 @@ class DataFile(object):
         columns = [None] * len(rows[0]) if not columns and rows else columns
         return (columns, rows)
 
+    def json(self) -> Dict:
+        """Load file content as Json object.
+
+        Returns
+        -------
+        dict
+        """
+        with self.load().open() as f:
+            return json.load(f)
+
     def load(self) -> FileHandle:
         """Get handle for the file.
 
         Returns
         -------
-        flowserv.model.files.base.FileHandle
+        flowserv.model.files.FileHandle
         """
         with self.service() as api:
             return api.runs().get_result_file(
