@@ -11,13 +11,13 @@
 from typing import List
 
 from flowserv.controller.serial.workflow.result import RunResult
+from flowserv.controller.worker.manager import WorkerPool
 from flowserv.model.workflow.step import WorkflowStep
-from flowserv.controller.worker.code import exec_func
-from flowserv.controller.worker.factory import WorkerFactory
+from flowserv.volume.manager import VolumeManager
 
 
 def exec_workflow(
-    steps: List[WorkflowStep], workers: WorkerFactory, rundir: str,
+    steps: List[WorkflowStep], workers: WorkerPool, volumes: VolumeManager,
     result: RunResult
 ) -> RunResult:
     """Execute steps in a serial workflow.
@@ -35,11 +35,11 @@ def exec_workflow(
     ----------
     steps: list of flowserv.model.workflow.step.WorkflowStep
         Steps in the serial workflow that are executed in the given context.
-    workers: flowserv.controller.worker.factory.WorkerFactory, default=None
+    workers: flowserv.controller.worker.manager.WorkerPool, default=None
         Factory for :class:`flowserv.model.workflow.step.ContainerStep` steps.
-    rundir: str, default=None
-        Working directory for all executed workflow steps.
-    result: flowserv.controller.worker.result.RunResult
+    volumes: flowserv.volume.manager.VolumeManager
+        Manager for storage volumes that are used by the different workers.
+    result: flowserv.controller.serial.workflow.result.RunResult
         Collector for results from executed workflow steps. Contains the context
         within which the workflow is executed.
 
@@ -48,17 +48,17 @@ def exec_workflow(
     flowserv.controller.worker.result.RunResult
     """
     for step in steps:
-        if step.is_function_step():
-            r = exec_func(step=step, context=result.context, rundir=rundir)
-        else:
-            worker = workers.get(step.image)
-            r = worker.exec(
-                step=step,
-                arguments=result.context,
-                rundir=rundir
-            )
+        # Get the worker that is responsible for executing the workflow step.
+        worker = workers.get(step)
+        # Prepare the volume store that is associated with the worker.
+        store = volumes.get(worker.volume)
+        volumes.prepare(store=store, inputs=step.inputs, outputs=step.outputs)
+        # Execute the workflow step and add the result to the overall workflow
+        # result. Terminate if the step execution was not successful.
+        r = worker.exec(step=step, context=result.context, store=store)
         result.add(r)
-        # Terminate if the step execution was not successful.
         if r.returncode != 0:
             break
+        # Update volume manager with output files for the workflow step.
+        volumes.update(store=store, files=step.outputs)
     return result
